;
;	plot_solar.pro
;
;	plot solar irradiance time series or spectrum using data from LISIRD full time series
;
;	USAGE: plot_solar, wave_or_day, type=type, file=file, mission=mission,
;			xrange=xrange, yrange=yrange, data=data,
;			proxy=proxy, ptitle=ptitle, pcolor=pcolor, 
;			planet=planet
;	where
;		wave_or_day = wavelength (nm) if type='TS' or 'CMP'
;			    = UARS Day if type='SP'
;
;		type = 'TS'  for Time Series (default if not given)
;		     = 'SP'  for SPectrum
;		     = 'CP'  for Comparison Plot between SOLAR and PROXY data
;
;		mission = 'SME', 'UARS', 'TIMED', or 'SORCE'
;
;		xrange = [ xmin, xmax ] for setting plot's X range
;			X range is time for Time Series (TS)
;			X range is wavelength for Spectrum (SP)
;			X range is time range for Comparison Plot (CP)
;
;		yrange = [ ymin, ymax ] for setting plot's Y range
;
;		file = NAME OF LISIRD SOLAR FULL TIME SERIES Text File (not required)
;
;		data = option to return data that was plotted
;			the "data" can be passed back to plot_solar as PROXY data
;
;		proxy = fltarr(2, *) of proxy data (or other SOLAR data)
;			proxy[0,*] = time - fractional year
;			proxy[1,*] = proxy data
;			ONLY valid if use the type = 'CP'
;
;		ptitle= title for Proxy plots
;
;		pcolor = color for Proxy data
;
;       planet = planet name or number (1-9) to time shift SOLAR data to planet
;
;
;	HISTORY:
;	2/5/07		Tom Woods	Original Code - based on plot_see.pro
;   6/7/07      Tom Woods   Updated with MISSION input option
;   8/15/08     Tom Woods   Updated so time series will work with 0.1 nm resolution data
;

pro plot_solar, wave_or_day, type=type, xrange=xrange, yrange=yrange, $
	file=file, mission=mission, data=data, $
	proxy=proxy, ptitle=ptitle, pcolor=pcolor, $
	planet=planet, debug=debug

;
;	give user HELP if called without parameters
;
if (n_params(0) lt 1) then begin
  print, ' '
  print, 'USAGE: plot_solar, wave_or_day, type=type, file=file, mission=mission, $'
  print, '                 xrange=xrange, yrange=yrange, data=data, $'
  print, '                 proxy=proxy, ptitle=ptitle, pcolor=pcolor, $'
  print, '                 planet=planet'
  print, ' '
  print, "    wave_or_day = wavelength if type='TS' or 'CP'"
  print, '                  (can be wavelength range)'
  print, "                = Day (YYYYDOY) if type='SP'"
  print, '                  (can be day range)'
  print, ' '
  print, '    file        = specify the text file for the LISIRD Solar Full Time Series'
  print, '                  (normally not needed as it will search for latest version)'
  print, ' '
  print, "    mission     = option to specify mission: 'SME','UARS','TIMED',or'SORCE'"
  print, ' '
  print, "    data        = output for this plot, fltarr(2,*) for 'TS' or 'SP'"
  print, "                                    or  fltarr(3,*) for 'CP' type"
  print, "    proxy       = fltarr(2,*) of proxy data used for type='CP'"
  print, '    ptitle      = title for PROXY plots'
  print, '    pcolor      = color for PROXY plots or color table for multiple SP plots'
  print, ' '
  print, '    planet      = planet name or number (1-9) to time shift SOLAR data to planet'
  print, ' '
  print, ' '
  wave_or_day = 121.5	; use default value to make some kind of plot
endif

;
;	plot based on "type"
;
ptype = 'TS'
if keyword_set(type) then ptype = strupcase(type)
if keyword_set(proxy) then ptype = 'CP'
if (not keyword_set(proxy)) and (ptype eq 'CP') then ptype='TS'

;
;	keep SOLAR data in common block
;	so this procedure doesn't have to read in data for every call
;
common  plot_solar_data, fname, dataname, windowset, solar, solarTime, solarYD
sfname = size(fname)
if (sfname[0] ne 0) or (sfname[1] ne 7) then begin
  fname=''
  dataname='?'
  windowset=0
endif

;
;	only read in data if it is needed
;		SORCE is default for 116-2400 nm and date > 2003/091
;		TIMED is default for 0-116 nm and date > 2002/039
;		UARS is default for 1991/276   < date < 2001/272  for 119-425 nm
;		SME is default for  1981/281 < date < 1989/103  for 115-303 nm
;
;	defines for "dataname"
;
dnames = [ 'SORCE', 'TIMEDSEE', 'UARS', 'SME' ]
dtimes = [ [2003091L, 2100000L], [2002039L, 2100000L], [1991276L, 2001272L], [1981281L,1989103L] ]
dwaves = [ [116,2400], [0,193], [119,425], [115,303] ]
dnum = n_elements(dnames) 

;
;	select the data (mission) based on input by date for 'SP' plot type
;	or by wavelength for other plot types (TS or CP)
;
if (strmid(ptype,0,1) eq 'S') then begin
  yd1 = long(wave_or_day[0])
  dselect = '?'
  for k=0,dnum-1 do begin
    if ((yd1 ge dtimes[0,k]) and (yd1 le dtimes[1,k])) then begin
      dselect = dnames[k]
      break
    endif
  endfor
endif else begin
  wave1 = wave_or_day[0]
  dselect = '?'
  for k=0,dnum-1 do begin
    if ((wave1 ge dwaves[0,k]) and (wave1 le dwaves[1,k])) then begin
      dselect = dnames[k]
      break
    endif
  endfor
endelse

dselect1 = dselect
if keyword_set(mission) then begin
  smission = strupcase(strmid(mission,0,2))
  if (smission eq 'SO') then dselect = dnames[0]
  if (smission eq 'TI') then dselect = dnames[1]
  if (smission eq 'UA') then dselect = dnames[2]
  if (smission eq 'SM') then dselect = dnames[3]
endif

if keyword_set(debug) then begin
  print, 'plot_solar: Using ', dselect, ' data set for ', ptype, ' plot.'
  if (dselect1 ne dselect) then print, '            Switched from default of ', dselect1
  if (dselect eq '?') then stop, 'STOP: check out error in finding data set...'
endif

;
;	check if need to re-read SOLAR data if "file" changes
;	compare file data type to "dataname" so don't have to reload same file
;
mustreadfile = 0
if keyword_set(file) then begin
  if (file ne fname) then begin
    pstar = strpos( file, '*' )
    if (pstar ge 8) then begin
      pslash = strpos( file, '/', /reverse_search )
      if (pslash lt -1) then pslash = -1
      fdataname = strmid(file,pslash+6,pstar-5)
      pdash = strpos( fdataname, '_' )
      if (pdash gt 0) then fdataname = strmid(fdataname,0,pdash)
      fdataname = strupcase(fdataname)
      if (fdataname ne dataname) then begin
        mustreadfile = 1
        fname = file
      endif
    endif else begin
      mustreadfile = 1
      fname = file
    endelse
  endif
endif

if (dselect ne '?') then begin
  selfname = 'full_' + strlowcase(dselect) + '_time_series*.txt'
endif else begin
  selfname = '?'
endelse
selfnamebase = strmid(selfname,0,9)
pslash = strpos( fname, '/', /reverse_search )
if (pslash lt -1) then pslash = -1
fnamebase = strmid(fname,pslash+1,9)
if (selfnamebase ne fnamebase) and (mustreadfile eq 0) then begin
  mustreadfile = 1
  fname = selfname
endif

;
;    ******    Read data file if not loaded yet
;
if (mustreadfile ne 0) then begin
  windowset = 0
  ;
  ;  first look for the SAVESET (*.sav) version
  ;
  filefound = 0
  trysave = 0
  fextpos = strpos( fname, '.', /reverse_search )
  if (fextpos lt 0) then begin
    fnamesave = fname + '.sav'
    trysave = 1
  endif else begin
    fext = strmid( fname, fextpos+1, strlen(fname) - fextpos - 1 )
    if (strupcase(fext) ne 'SAV') then begin
      fnamesave = strmid(fname,0,fextpos) + '.sav'
      trysave = 1
    endif
  endelse
  if (trysave ne 0) then begin
    filelist = findfile( fnamesave, count=fcount )
    if (fcount lt 1) then begin
       ;  try from $solar_data directory
       fdir = getenv( 'solar_data' )
	   if (strlen(fdir) gt 0) then begin
	      fsearch = fdir + '/' + fnamesave
	      filelist = findfile( fsearch, count=fcount )
	   endif else fcount = 0
	endif
    if (fcount ge 1) then begin
      fname = filelist[fcount-1]
	  file = fname
      print, 'plot_solar: Restoring SAVESET: ', fname, ' ...'
      restore, fname    ; restore 'solar' from SAVESET
      filefound = 1
    endif
  endif
  ;
  ;  if no SAVESET then read ascii text file
  ;
  if (filefound eq 0) then begin
	filelist = findfile( fname, count=fcount )
	if (fcount lt 1) then begin
	  fdir = getenv( 'solar_data' )
	  if (strlen(fdir) gt 0) then begin
	    fsearch = fdir + '/' + fname
	    filelist = findfile( fsearch, count=fcount )
	  endif else fcount = 0
	  if (fcount lt 1) then begin
	    print, 'plot_solar: ERROR - can not find data file - ', fname
	    fname = ''
	    return
	  endif
	endif
	fname = filelist[fcount-1]	; pick latest version of file
	file = fname
	print, 'plot_solar: Reading Data File: ', fname, ' ...'
	solar = read_lisird_datafile( fname )
	filefound = 1
	print, ' '
	;
	;  do SAVESET so faster read next time
	;
	trysave = 0
    fextpos = strpos( fname, '.', /reverse_search )
    if (fextpos lt 0) then begin
      fnamesave = fname + '.sav'
      trysave = 1
    endif else begin
      fext = strmid( fname, fextpos+1, strlen(fname) - fextpos - 1 )
      if (strupcase(fext) ne 'SAV') then begin
        fnamesave = strmid(fname,0,fextpos) + '.sav'
        trysave = 1
      endif
    endelse
    if (trysave ne 0) then begin
      print, 'plot_solar: Making SAVESET: ', fnamesave
      save, solar, file=fnamesave
      fname = fnamesave
      file = fname
    endif
  endif
  ;
  ;	get "dataname"
  ;
  pslash = strpos( fname, '/', /reverse_search )
  if (pslash lt -1) then pslash = -1
  dataname = strmid(fname,pslash+6,10)
  pdash = strpos( dataname, '_' )
  if (pdash gt 0) then dataname = strmid(dataname,0,pdash)
  dataname = strupcase(dataname)
  ;
  ;	make "solarTime" for each new file read
  ; make as fraction of year
  ; 		leap year is either (divisible by 4 and not divisible by 100) or divisible by 400
  ;
  if keyword_set(debug) then print, 'plot_solar: Converting data time to fractional year...'
  yrdays = 365.D0
  solarYD = jd2yd( solar.julian )
  year = long(solaryd / 1000L) 
  yrisleap = lonarr(n_elements(year))
  wleap=where( ( ((year mod 4) eq 0) and ((year mod 100) ne 0) ) or ((year mod 400) eq 0), n_true)
  if n_true gt 0 then yrisleap[wleap] = 1
  solarTime = year + (long((solaryd-1) mod 1000L)+0.5D0) / (yrdays + yrisleap)
endif

;
;  verify 'solar' data set is loaded
;
ssolar = size(solar)
if (ssolar[0] ne 1) or (ssolar[2] ne 8) then begin
  print, 'plot_solar:  ERROR - no "solar" data loaded. Exiting...'
  if keyword_set(debug) then stop, 'STOP: check out "solar" data issue...'
  return
endif

;
;	keep Planet information in common block 
;	so don't have to re-calculate planet position on every call
;	unless the planet changes
;
common  plot_solar_planet_info, lastplanet, lastdataname, planetdist2, planetshift, planetstr
psize = size(lastplanet)
if (psize[psize[0]+1] eq 0) then begin
  lastplanet=-1
  lastdataname='?'
  planetstr = '?'
endif
if keyword_set(planet) then begin
  planetnum = 0L
  psize = size(planet)
  if (psize[1] eq 7) then begin
	  ;  Figure out which Planet string was given
	  pstr = strupcase(strmid(planet,0,3))
	  pall = [ "SUN", "MER", "VEN", "EAR", "MAR", "JUP", "SAT", "URA", "NEP", "PLU" ]
	  planetnum = where( pall eq pstr )
  endif else planetnum = planet
  if (planetnum ne lastplanet) or (dataname ne lastdataname) then begin
    print, 'plot_solar: calculating planet position (a few seconds) ...'
    if (planetnum lt 1) or (planetnum gt 9) then planetnum = 3	; Earth
    lastplanet = planetnum
    lastdataname = dataname
    case planetnum of
      1: planetstr = 'Mercury'
      2: planetstr = 'Venus'
      3: planetstr = 'Earth'
      4: planetstr = 'Mars'
      5: planetstr = 'Jupiter'
      6: planetstr = 'Saturn'
      7: planetstr = 'Uranus'
      8: planetstr = 'Neptune'
      9: planetstr = 'Pluto'
      else: planetstr = '?'
    endcase
    ndates = n_elements(solar.julian)
    planetdist2 = fltarr(ndates)
    planetshift = fltarr(ndates,2,2)
    daymin = long(min(solar.julian - 0.5))
    daymax = long(max(solar.julian - 0.5))
    solar_day = long(solar.julian - 0.5)
    numuniquedates = long( daymax - daymin  + 1.)
    for pdate=daymin,daymax,1L do begin
      wgoodp = where( (solar_day eq pdate), numgoodp )
      if (numgoodp gt 0) then begin
        earth2planet, planetnum, pdate, pdist, pshift
        ; division correction is R^2
        planetdist2[wgoodp] = pdist^2.    
        ;  force time shifts to be integers
        pshift[0,0] = round(pshift[0,0])
        pshift[1,0] = round(pshift[1,0])
        planetshift[wgoodp,0,0] = pshift[0,0]
        planetshift[wgoodp,0,1] = pshift[0,1]
        planetshift[wgoodp,1,0] = pshift[1,0]
        planetshift[wgoodp,1,1] = pshift[1,1]
 	    ;
    	;  fix end points so shifting will not crash program
	    ;
	    if (pdate le (daymin+27L)) then begin
	      kmin = long(pdate-daymin-pdate+1)
	      wbad = where( planetshift[wgoodp,0,0] lt kmin, numbad )
	      if (numbad gt 0) then planetshift[wgoodp[wbad],0,0] = kmin
          wbad = where( planetshift[wgoodp,1,0] lt kmin, numbad )
          if (numbad gt 0) then planetshift[wgoodp[wbad],1,0] = kmin	    
	    endif
	    if (pdate ge (daymax-27L)) then begin
	      kmax = long(daymax-pdate-1)
          wbad = where(planetshift[wgoodp,0,0] gt kmax, numbad) 
          if (numbad gt 0) then planetshift[wgoodp[wbad],0,0] = kmax
          wbad = where(planetshift[wgoodp,1,0] gt kmax, numbad) 
          if (numbad gt 0) then planetshift[wgoodp[wbad],1,0] = kmax
        endif    
      endif
    endfor
    if keyword_set(debug) then stop, 'STOP: check out planet shift results...'
  endif
endif

;
;	common stuff for making the plots
;
;!fancy=4
!psym=10
data = 0.0	; empty data set
yrdays = 365.D0

mtitle = dataname
if keyword_set(planet) then begin
  mtitle = planetstr + ' - ' + mtitle
endif

if (ptype eq 'SP') then begin
	;
	;   ******    Plot SPECTRUM
	;
	if (windowset eq 0) then begin
	  window, retain=2
	  windowset = 1
	endif
	temp=min( abs(solarYD - (wave_or_day[0]+0.5)), wgoods )
	theDay = solarYD[wgoods]
	dayStr = strtrim( long(theDay/1000L), 2 )+'/'+strtrim(long(theDay mod 1000L),2)
	factor = 1.0
	wgood = where( (solarYD eq theDay) and (solar.irradiance gt 0), numgood )
	if (numgood le 0) then begin
	  print, 'plot_solar: WARNING - no valid spectral data found'
	  return
	endif
	if keyword_set(planet) then begin
	  factor = 1. / mean(planetdist2[wgood])
	  ;  get day offsets for planet time shifting
	  longTime = long(solar.julian)
	  jdinput1 = long(yd2jd( wave_or_day[0] ) + planetshift[wgoods,0,0])
	  temp=min( abs(longTime - jdinput1), wgood1 )
	  jd1 = longTime[wgood1]
	  wgood1 = where( (longTime eq jd1) and (solar.irradiance gt 0), numgood1 )
	  jdinput2 = long(yd2jd( wave_or_day[0] ) + planetshift[wgoods,1,0])
	  temp=min( abs(longTime - jdinput2), wgood2 )
	  jd2 = longTime[wgood2]
	  wgood2 = where( (longTime eq jd2) and (solar.irradiance gt 0), numgood2 )
	  if (numgood1 le 0) and (numgood2 le 0) then begin
	    print, 'plot_solar: WARNING - no valid spectral data found'
	    return
	  endif
	  spcnt = 0L
	  spwave1 = 0. & spflux1 = 0.
	  if (numgood1 gt 0) then begin
	    spwave1 = reform((solar.wave)[wgood1])
	    spflux1 = reform((solar.irradiance)[wgood1]) * 1000. * factor
		spcnt = spcnt + 1L
	  endif
	  spwave2 = 0. & spflux2 = 0.
	  if (numgood2 gt 0) then begin
	    spwave2 = reform((solar.wave)[wgood2])
	    spflux2 = reform((solar.irradiance)[wgood2]) * 1000. * factor
	    if (spcnt eq 1L) then begin
	      spflux2 = interpol( spflux2, spwave2, spwave1 )
	      spwave2 = spwave1
	    endif
		spcnt = spcnt + 1L
	  endif
	  spwave = (spwave1 + spwave2)/float(spcnt)
	  if (spcnt eq 2L) then begin
	    spflux = (spflux1 * mean(planetshift[wgood1,0,1]) + spflux2 * mean(planetshift[wgood1,1,1]))
	  endif else begin
	    spflux = (spflux1 + spflux2)/float(spcnt)
	  endelse
	endif else begin
	  spwave = reform((solar.wave)[wgood])
	  spflux = reform((solar.irradiance)[wgood]) * 1000. * factor
	endelse
	
	;
	;   average together several spectra if day range given
	;
	if (n_elements(wave_or_day) gt 1) then begin

	  temp=min( abs(solarYD - (wave_or_day[1]+0.5)), wgoods2 )
	  theDay2 = solarYD[wgoods2]
	  dayStr = dayStr + ' - ' + $
	        strtrim( long(theDay2/1000L), 2 )+'/'+strtrim(long(theDay2 mod 1000L),2)
	  factor = 1.0
	  longTime = long(solar.julian)
	  kjd1 = long(longTime[wgoods]) + 1L
	  kjd2 = long(longTime[wgoods2])
	  if (kjd1 lt kjd2) then kstep = 1L else kstep = -1L
	  cnt = 1L  ; already have 1 spectrum in spflux
	  
	  for kjd = kjd1, kjd2, kstep do begin
 	    wgood = where( (longTime eq kjd) and (solar.irradiance gt 0), numgood )
	    if (numgood gt 1) then begin
 	      if keyword_set(planet) then begin
	        factor = 1. / mean(planetdist2[wgood])
	        ;  get day offsets for planet time shifting
	        jdinput1 = kjd + planetshift[wgood[0],0,0]
	        temp=min( abs(longTime - jdinput1), wgood1 )
	        jd1 = longTime[wgood1]
	        wgood1 = where( (longTime eq jd1) and (solar.irradiance gt 0), numgood1 )
	        jdinput2 = kjd + planetshift[wgood[0],1,0]
	        temp=min( abs(longTime - jdinput2), wgood2 )
	        jd2 = longTime[wgood2]
	        wgood2 = where( (longTime eq jd2) and (solar.irradiance gt 0), numgood2 )
	 	    spcnt = 0L
	        spwave1 = 0. & spflux1 = 0.
	        if (numgood1 gt 0) then begin
	          spwave1 = reform((solar.wave)[wgood1])
    	      spflux1 = interpol(reform((solar.irradiance)[wgood1]) * 1000. * factor, spwave1, spwave)
	          spcnt = spcnt + 1L
	        endif
	        spwave2 = 0. & spflux2 = 0.
	        if (numgood2 gt 0) then begin
	          spwave2 = reform((solar.wave)[wgood2])
	          spflux2 = interpol(reform((solar.irradiance)[wgood2]) * 1000. * factor, spwave2, spwave)
	    	  spcnt = spcnt + 1L
	        endif
	        if (spcnt eq 2L) then begin
	          spflux3 = (spflux1 * mean(planetshift[wgood1,0,1]) + spflux2 * mean(planetshift[wgood1,1,1]))
	          spflux = spflux + spflux3
	          cnt = cnt + 1L
	        endif else if (spcnt gt 0) then begin
	          spflux3 = (spflux1 + spflux2)/float(spcnt)
	          spflux = spflux + spflux3
	          cnt = cnt + 1L
	        endif
	      endif else begin
	        spwave2 = reform((solar.wave)[wgood])
	        spflux = spflux + interpol(reform((solar.irradiance)[wgood]) * 1000. * factor, spwave2, spwave )
	        cnt = cnt + 1L
	      endelse
	    endif
	  endfor

	  if keyword_set(debug) then print, 'plot_solar: average of ', strtrim(cnt,2), ' days for spectrum'
	  spflux = spflux / float(cnt)	  
	endif

	if keyword_set(xrange) then theXrange=xrange else begin
	  theXrange=[ long(min(spwave-20)*10.)/10L, long(max(spwave+20)*10.)/10L ]
	endelse
    if keyword_set(yrange) then theYrange=yrange else begin
      theYmax = 10.^round(alog10(max(spflux)))
      if (theYmax lt max(spflux)) then theYmax = theYmax * 3.
      theYmin = 10.^round(alog10(min(spflux)/3.))
      theYrange = [theYmin, theYmax]
    endelse
	plot_io, spwave, spflux, xrange=theXrange, xstyle=1, $
	    yrange=theYrange, ystyle=1, xtitle='Wavelength [nm]', $
	    ytitle='Irradiance [mW/m!U2!N/nm]', $
	    title=mtitle+': '+dayStr, thick=2, charthick=1.5, xmargin=[11,3]
	wwv = where( (spwave ge theXrange[0]) and (spwave le theXrange[1]), numwv)
	if (numwv gt 0) then begin
		data = dblarr(2,numwv)
		data[0,*] = spwave[wwv]
		data[1,*] = spflux[wwv]
	endif
	
endif else begin
	;
	;   ******    Time Series Plot OR Comparison Plot with PROXY data
	;	                  Both have Time Series Plot
	;
	;	Updated (8/15/08 T. Woods) to allow for 0.1 nm resolution data
	;	but still searches for 1-nm intervals for integrations
	;
	if (windowset eq 0) then begin
	  window, retain=2
	  windowset = 1
	endif
	if (ptype eq 'CP') then !p.multi=[0,1,2]
	;
	;	first plot (top) is TIME SERIES
	;
	factor = 1.0
	theWave = long(wave_or_day[0]) + 0.5
	temp=min( abs(solar.wave - theWave), wgood )
    theWave = (solar.wave)[wgood]
    waveStr = string( theWave, format='(F6.1)' )
    wgood = where( (solar.wave gt (theWave - 0.05)) and $
	    			(solar.wave lt (theWave + 0.05)), numgood )
    if (numgood gt 1) then begin
	  wpos = where( (solar.irradiance)[wgood] gt 0, numpos )
	endif else numpos = 0
	if (numpos le 1) then begin
	  print, 'plot_solar: WARNING - no valid data found at ', wavestr, ' nm'
      return
	endif
	theTime = solarTime[wgood[wpos]]    
 	if keyword_set(planet) then begin
 	  factor = 1. / planetdist2[wgood[wpos]]
	  allFlux = interpol((solar.irradiance)[wgood[wpos]], theTime, $
	  			theTime+long(planetshift[wgood[wpos],0,0])/yrdays) * planetshift[wgood[wpos],0,1] $
			+ interpol((solar.irradiance)[wgood[wpos]], theTime, $
				theTime+long(planetshift[wgood[wpos],1,0])/yrdays) * planetshift[wgood[wpos],1,1]
	endif else begin
	  allFlux = (solar.irradiance)[wgood[wpos]]
	endelse
	;  convert to mW/m^2/nm
     allFlux = reform(allFlux) * 1000. * factor
    theYtitle = 'Irradiance [mW/m!U2!N/nm]'
    theTitle = mtitle+' at '+waveStr+' nm'
    if (n_elements(wave_or_day) gt 1) then begin
       ;  integrate over more wavelengths
       theWave = long(wave_or_day[0])
       theWave2 = long(wave_or_day[1])
       if (theWave2 eq theWave) then theWave2 = theWave + 1
       if (theWave gt theWave2) then begin
         tempWave = theWave
         theWave = theWave2
         theWave2 = tempWave
       endif
       waveStr = strtrim( theWave, 2 )
       waveStr2 = strtrim( theWave2, 2 )
       numWave = long(abs(theWave2-theWave)) * 10L	; search on 0.1 nm grid
       if numWave gt 0 then begin
         k1 = float(theWave)
         k2 = float(theWave2)
         kstep = 0.1
         allFlux[*] = 0.0	; zero out "allFlux" and re-integrate over band
         cnt = 0.0
         binWave = fltarr(numWave)
         for kwv=k1,k2,kstep do begin	           
    	    wgood2 = where( (solar.wave gt kwv) and $
    			(solar.wave lt (kwv + 0.1)), numgood2 )
    		if (numgood2 gt 1) then begin
		      wpos2 = where( (solar.irradiance)[wgood2] gt 0, numpos2 )
		      if (numpos2 gt 1) then begin
		        binWave[long((kwv-k1)/kstep)] = solar.wave[wgood2[wpos2[0]]]
			    theTime2 = solarTime[wgood2[wpos2]]    
		        if keyword_set(planet) then begin
			      tempFlux = interpol((solar.irradiance)[wgood2[wpos2]], theTime2, $
	  					theTime+long(planetshift[wgood[wpos],0,0])/yrdays) * planetshift[wgood[wpos],0,1] $
					+ interpol((solar.irradiance)[wgood2[wpos2]], theTime2, $
						theTime+long(planetshift[wgood[wpos],1,0])/yrdays) * planetshift[wgood[wpos],1,1]
			    endif else begin
			      tempFlux = interpol( (solar.irradiance)[wgood2[wpos2]], theTime2, theTIme ) 
			    endelse
			    allFlux = allFlux + reform(tempFlux) * 1000. * factor
			    cnt = cnt + 1.
			  endif
			endif
         endfor
         ;  Normalize integration for desired band width =  avgband * (theWave2-theWave) / (avgband * cnt)
         wgdwv = where( binWave gt 0, numgdwv )
         if (numgdwv gt 1) then begin
           avgband = abs(binWave[wgdwv[numgdwv-1]] - binWave[wgdwv[0]]) / (numgdwv-1.)
         endif else begin
           avgband = 1.
         endelse
         if keyword_set(debug) then print, 'plot_solar: ', strtrim(long(cnt),2), $
         	' wavelengths averaged with band width of ', strtrim(avgband,2)
         allFlux = allFlux * (theWave2-theWave) / float(cnt)
         theYtitle = 'Irradiance [mW/m!U2!N]'
         theTitle = mtitle+': '+waveStr+'-'+waveStr2+' nm'
       endif
    endif
	;  FIX bad data
	wbad = finite(allFlux)
	wfix = where(wbad eq 0)
	if (wfix[0] ne -1) then allFlux[wfix] = 0.0
	wthere = where( allFlux gt 0. )
	if (wthere[0] eq -1) then begin
	   print, 'plot_solar: WARNING - no valid time series data found'
	   return
	endif
	allFlux = allFlux[wthere]
	theTime = theTime[wthere]
	
	if keyword_set(xrange) then theXrange=xrange else begin
	  theXrange=[long(min(theTime)),long(2.*(max(theTime)+0.5))/2.]
	endelse
	if keyword_set(yrange) then theYrange=yrange else begin
	  ymin = min(allFlux)
	  ymax = max(allFlux)
	  yscale = ymax - ymin
	  ymid = (ymax+ymin)/2.
	  theYrange = [ (ymid - yscale) > 0., ymid + yscale ]
	endelse
    if (ptype eq 'CP') then ymargin=[2,2] else ymargin=[4,2]
    ;  
    ;   plot the time series
    ;
	plot, theTime, allFlux,  xrange=theXrange, xstyle=1, $
	    yrange=theYrange, ystyle=1, thick=2, xtitle='Time [year]', ytitle=theYtitle, $
	    title=theTitle, xmargin=[12,3], ymargin=ymargin
	;
	;	second plot (bottom) is comparison to PROXY data
	;
	if (ptype ne 'CP') then begin
	  data = dblarr(2,n_elements(allFlux))
	  data[0,*] = theTime
	  data[1,*] = allFlux
	endif else begin
	  if (not keyword_set(ptitle)) then ptitle='Proxy Data'
	  proxyFlux = interpol( proxy[1,*], proxy[0,*], theTime )
	  wthere2 = where( (allFlux gt 0.) and (proxyFlux gt 0.) $
	        and (theTime ge theXrange[0]) and (theTime le theXrange[1]) )
	  if (wthere2[0]) ne -1 then begin
	    ;  overplot proxy data first
	    cfit = poly_fit( proxyFlux[wthere2], allFlux[wthere2], 1, yfit = pfit )
	    if (keyword_set(pcolor)) then oplot, theTime[wthere2], pfit, color=pcolor $
	    else oplot, theTime[wthere2], pfit, line=2
	    
	    plot, proxyFlux[wthere2], allFlux[wthere2], psym=3, thick=3, $
	        xstyle=1, ystyle=1, xtitle=ptitle, $
	        ytitle=theYtitle, title='', xmargin=[12,3], ymargin=[3,1.5]
	    ; if (keyword_set(pcolor)) then oplot,proxyFlux[wthere2], allFlux[wthere2], $
	    ;	psym=3, color=pcolor
	    
	       data = dblarr(3,n_elements(wthere2))
	       data[0,*] = theTime[wthere2]
	       data[1,*] = allFlux[wthere2]
	       data[2,*] = proxyFlux[wthere2]
	  endif
	  !p.multi=0
	endelse
endelse

if keyword_set(debug) then stop, 'Check out results at end of plot_solar...'

return
end
